/*
    NewTemplateCmd.cc - template creation

    Will 18, 2001: will@opaline.demon.co.uk
*/

#include <DMenuCmd.h>
#include <FileSelect.h>
#include <LineEdit.h>
#include <FileAccess.h>
#include "OpResources.h"
#include "NewTemplateCmd.h"
#include "AppWindow.h"
#include <MessageDialogManager.h>
#include <InfoDialogManager.h>
#include <WorkingDialogManager.h>
#include "wave/waveclass.h"
#include "CreateTemplateFileCmd.h"


NewTemplateCmd::NewTemplateCmd ( char *name, int active, int dataType ) : NoUndoCmd ( name, active )
{
	_menuRealised = False;
	_srcSel = False;
	_dstSel = False;
	_dataType = dataType;
	_name = name;
	_waveNum = 0;
}


void NewTemplateCmd::doCommand()
{
	if ( _menuRealised == True )
		return;

	unsigned int n;
	unsigned int maxNameLength = 0;
	char title[22];
	char subtitle[26];
		
	if ( _dataType == 0 )
	{
		strcpy ( title, "New integer template" );
		strcpy ( subtitle, "Integer data size source?" );
	}
	else
	{
		strcpy ( title, "New float template" );
		strcpy ( subtitle, "Float data size source?" );
	}

	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 18 )
		maxNameLength = 18;	// space for non-wave buttons below

	_arrayItems += 3;	// allow for three extra buttons below
		
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 3 ); n++ )
		strcpy ( ( char * ) &strings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( strings [ n ], "Select wave file" );
	strcpy ( strings [ n+1 ], "User defined size" );
	strcpy ( strings [ n+2 ], "Quit" );

	_dMenu = new DMenuCmd ( title, TRUE, TRUE,
								&NewTemplateCmd::dMenuSelectionCallback,
								( void * ) this, subtitle,
								_arrayItems, maxNameLength, ( char * ) &strings );

	_src = new FileSelect ( "Select_wave" , TRUE,
							&NewTemplateCmd::sourceSelectedCallback, NULL,
							&NewTemplateCmd::sourceCancelledCallback,
							( void * ) this );
	_srcSel = True;

	_dst = new FileSelect ( "Template_file", TRUE,
							&NewTemplateCmd::targetSelectedCallback, NULL,
							&NewTemplateCmd::targetCancelledCallback,
							( void * ) this );
	_dstSel = True;

	_edit = new LineEdit ( "Enter data size", TRUE,
							&NewTemplateCmd::sizeEnteredCallback,
							&NewTemplateCmd::sizeCancelledCallback,
							( void * ) this );

	_menuRealised = True;
	_dMenu->execute();
}


void NewTemplateCmd::dMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;				

	if ( ( selectedItemNum == obj->_arrayItems || selectedItemNum == 0 ) )	// 'quit menu' selected or WM close
	{
		obj->_menuRealised = False;	// menu will be popped down
		delete obj->_src;
		delete obj->_dst;
		delete obj->_edit;
	}
	else
		obj->newTemplate ( selectedItemNum );

	obj->_menuRealised = False;
	delete obj->_dMenu;
}


void NewTemplateCmd::newTemplate ( unsigned int selectedItemNum )
{
	for ( int count = 0; count < wavePak->Count(); count++ )
	{
		if ( (*wavePak)[count].pWave->waveNum() == selectedItemNum )
		{
			if ( (*wavePak)[count].fLock == ACTIVE )
				return;
			else
			{
				_waveNum = selectedItemNum;
				(*wavePak)[count].fLock = ACTIVE;
			}
		}
	}

	if ( ( selectedItemNum > 0 ) && ( selectedItemNum <= ( unsigned int ) wavePak->Count() ) )
	{
		if ( (*wavePak)[selectedItemNum-1].pWave->dataLength() == 0 )
		{
			char buf[BUFSIZE];
			snprintf ( buf, sizeof ( buf ), "%s\nSource file is empty, can't create zero-length template.",
						(*wavePak)[selectedItemNum-1].pWave->name() );
			buf [ sizeof ( buf ) - 1 ] = 0;
			messageDialogManager->post ( buf );
			_menuRealised = False;
			return;
		}		

		// create template from wave on menu; delete edit, src - not needed for input
		delete _edit;
		_srcSel = False;
		delete _src;

		createTemplate ( (*wavePak)[selectedItemNum-1].pWave->totalSamples() );
	}
	else
	{
		if ( selectedItemNum == ( ( unsigned int ) wavePak->Count() + 1 ) ) 
			_src->execute();	// select wave file
		else
		{
			if ( selectedItemNum == ( ( unsigned int ) wavePak->Count() + 2 ) ) 
			{	// user-defined size
				char buf[30];
				char help[256];

				/* retrieve resource to be displayed */
		 		snprintf ( buf, sizeof ( buf ), "%d", mixwaveResources->defaultTemplateSize() );
				buf [ sizeof ( buf ) - 1 ] = 0;	
				snprintf ( help, sizeof ( help ), "Specifies number of data points in template: default %d",
							mixwaveResources->defaultTemplateSize() ); 
				help [ sizeof ( help ) - 1 ] = 0;

				_edit->setColumns ( strlen ( buf ) );	// set width before instantiation
				_edit->setHelp ( help );
				_edit->execute();
				_edit->setText ( buf );	// can't set text before instantiation...
			}
		}
	}
}


void NewTemplateCmd::sourceSelectedCallback ( void *clientData, char *filename )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;

	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
		obj->sourceSelected ( filename );
	else
		obj->_menuRealised = False;

	delete obj->_src;
	delete obj->_edit;	// not needed, will pop up target file selector instead
}


void NewTemplateCmd::sourceCancelledCallback ( void *clientData )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;

	obj->_menuRealised = False;
	delete obj->_src;
	delete obj->_dst;
	delete obj->_edit;
	(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
}


void NewTemplateCmd::sourceSelected ( char *filename )
{
	Wave *pWave = new Wave;
	_srcSel = False;
	
	if ( !pWave->LoadHeader ( filename ) )
	{
    	char buf[BUFSIZE];
			strncpy ( buf, pWave->name(), sizeof ( buf ) );
			buf [ sizeof ( buf ) - 1 ] = 0;
			strncat ( buf, "\nWave::LoadHeader failed.", sizeof ( buf ) - strlen ( buf ) - 1 );
			messageDialogManager->post ( buf );
			delete pWave;
			_menuRealised = False;
			return;
	}
	else
		createTemplate ( pWave->totalSamples() );
		
	delete pWave;                                                            
}


void NewTemplateCmd::sizeEnteredCallback ( void *clientData, char *text )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;
	
	int num;
	sscanf ( text, "%d", &num );	

	if ( num > 0 )
		obj->createTemplate ( num );

	delete obj->_edit;
}


void NewTemplateCmd::sizeCancelledCallback ( void *clientData )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;

	obj->_menuRealised = False;
	delete obj->_edit;
	delete obj->_src;
	delete obj->_dst;
	(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
}


void NewTemplateCmd::createTemplate ( unsigned int numSamples )
{
	_numSamples = numSamples;
	_dst->execute();
}


void NewTemplateCmd::targetSelectedCallback ( void *clientData, char *filename )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;

	obj->_filename = filename;

	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
		obj->createTemplateFile();
	else
		obj->_menuRealised = False;

	delete obj->_dst;

	if ( obj->_srcSel == True )
		delete obj->_src;
}


void NewTemplateCmd::targetCancelledCallback ( void *clientData )
{
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;

	obj->_menuRealised = False;
	delete obj->_dst;

	if ( obj->_srcSel == True )
		delete obj->_src;

	(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
}


void NewTemplateCmd::createTemplateFile()
{
	CreateTemplateFileCmd *ctfCmd = new CreateTemplateFileCmd (
								"CreateTemplate", TRUE, _filename, _numSamples,
								mixwaveResources->blockLength(), _dataType );

	ctfCmd->execute ( &NewTemplateCmd::createTemplateFinishedCallback, ( void * ) this );

	_menuRealised = False;
}


void NewTemplateCmd::createTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *ctfObj = ( CreateTemplateFileCmd * ) cmd;
	NewTemplateCmd *obj = ( NewTemplateCmd * ) clientData;
	
	if ( interrupted )
		messageDialogManager->post ( "Interrupted!" );
	else
		obj->createTemplateFinished ( ctfObj );
		
	delete ctfObj;
	(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
}


void NewTemplateCmd::createTemplateFinished ( CreateTemplateFileCmd *ctfObj )
{
	if ( mixwaveResources->enableMessagePopups() == True )
	{
		if ( ctfObj->error() == 0 )
		{
			char msg[BUFSIZE];
			char type[8];
		
			if ( _dataType == 0 )
				strcpy ( type, "Integer" );
			else
				strcpy ( type, "Float" );
			
			snprintf ( msg, sizeof ( msg ), "%s\nNew %s template written.",
						ctfObj->target(), type );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}
