/*
    ViewerHelpCmd.cc

    Will Eighteen, 2006: will@opaline.demon.co.uk
*/

#include "ViewerHelpCmd.h"
#include "Application.h"
#include <TextEdit.h>


ViewerHelpCmd::ViewerHelpCmd ( const char *name, int active ) :  NoUndoCmd ( name, active )
{
	_editRealised = FALSE;
}


ViewerHelpCmd::~ViewerHelpCmd()
{
	// empty
}


void ViewerHelpCmd::doCommand()
{
	if (_editRealised == TRUE )
		return;

	_edit = new TextEdit ( ( char * ) name(), TRUE, &ViewerHelpCmd::okCallback, &ViewerHelpCmd::cancelCallback, ( void * ) this );	
	_edit->setRows ( 19 );
	_edit->setColumns ( 70 );
	_editRealised = TRUE;
	_edit->execute();
	_edit->setText ( "The mixwave viewer/editor is divided into four \
sections:\n\n1) the display pane;\n2) a function button pane to the right;\n\
3) a dynamic scrollbar below the display pane;\n4: a data display pane \
below the scrollbar.\n\nIf a wave file is being viewed, in addition to the \
main window comprising the four panes above, there is a supplementary data \
display popup which gives the same information as the data display pane of \
the viewer but in different format. The former shows data in samples, the \
latter in time (mins:decimal seconds).\nChanges made to the data in either of \
the data display panes are reflected in the other.\n\nThe supplementary data \
display pane may be closed independently of the view window, but cannot be re-\
invoked without closing the view window and re-selecting the wave view.\n\n\n\
1) The display pane.\nThis is the visual area, used for display and editing of \
waves and templates (files containing data in integer or floating-point \
format).\n\nIn this pane, the left mouse button depressed in the pane and \
dragged horizontally selects a region. If the drag action is continued outside \
the left or right pane boundaries, any portion of the file not already \
already visible will be dragged into the view (and loaded into memory if not \
already loaded). Note that right-dragging preserves the start point at \
the left of the pane and left-dragging the end point at the right of the \
pane.\n\nThe left mouse button depressed and dragged vertically in \
the pane controls scaling. The central third of the view is an inactive \
region for this operation. Scaling only begins when the click-and-drag has \
either passed outside or occurs outside of this area. Upwards motion \
increases scaling, downwards decreases it. Since mouse click operations are \
required for other operations such as selection, the scale may be locked once \
established. To restore the scaling to unity, click and drag vertically by a \
small amount inside the central third of the pane height.\n\nIf the right \
mouse button is depressed and the mouse dragged from left to right whilst \
also dragging vertically, the file is edited. The data in the view pane \
changes to reflect this. In essence, this operation draws data. It is \
important to note that the click-and-drag changes the file data directly. \
There is no undo. Either a copy of the file to be edited should be used or \
the file permissions should be changed to read-only if it desired not to \
corrupt the original data. In this latter case, although the data in the view \
pane can be changed, these changes will be discarded as soon as the window is \
redrawn by a file access operation such as left-click-and-drag, 'Prev(ious)' \
or 'Next'.\n\nNote that file viewing can cause a delay whilst the file loads. \
This is most evident after 'State / Load' and then 'File / View' since the \
entire file is loaded into memory. Loading a file in the normal way via \
'File / Load' does not suffer lag since only a mix buffer size segment is \
initially loaded.\n\n\n2) The function button pane\n\nThere are three groups \
of items: operation buttons, a scale display and view lock toggle \
buttons.\n\nOperation buttons\n\nOk\nThis closes the view, preserving the \
current values in the data display pane and the contents of the view pane.\
\n\nCancel\nThis closes the view popup and discards the current data \
settings. Any selection currently made will be lost. The amount of the file \
currently in view is preserved, but on redisplay this memory usage will be \
reallocated to the start of the file, so an initial segment will be shown \
corresponding to the size of the last segment in the view when 'Cancel' was \
activated.\n\nLine\nToggles line drawing / point \
plotting. Plot mode is most useful at high resolution in small sections of \
the file, where individual samples need to be located. Line mode connects \
data points, making the overall shape easier to see.\nThe button label \
toggles between 'Line' and 'Point' when clicked to show the current mode,\
\n\nZoom in\nThis expands a \
selected region to fit the view pane width. The button label will toggle to \
'Zoom out' to show the zoom is active. The scrollbar trough displays the \
highlight colour when a selection is made in the file; this serves a reminder \
that a selected region exists even if it is not currently visible. Note that \
when a view selection is cleared, the scrollbar trough remains highlighted \
unless the mouse is left-clicked on the scrollbar.\n\nAll\nThis displays the \
entire file. Note that the time taken to display it will depend upon how much \
of it is currently loaded into memory. An initial 'All' \
may take a few seconds. 'All' following 'Zoom in' may be \
virtually instantaneous.\nTo find out how much of a file is loaded into the \
view pane, left-click on the scrollbar: its size remains unchanged if all \
that is loaded is currently in the view; the length of the scroll\
bar in proportion to its trough length indicates the amount of the file \
loaded.\n\nNext\nDisplays the next 'page' \
in the file: a 'page' is the amount of the file currently in the view pane. \
If there is no selection in the file and the scrollbar trough is not \
highlighted then the page increment remains constant, otherwise it dynamically \
resizes so that 'Next' displays consecutive sections of highlighted length \
regardless of whether the selection initially occupied only part of the view \
pane. The size of the scrollbar tracks this behaviour. On returning to the \
highlighted region with 'Prev' so it is visible in the view pane, the scroll\
bar returns to its original size.\n\nPrev\nViews the previous page; the \
dynamic scrollbar behaviour is as for 'Next'. Note that dragging the scrollbar \
may itself be a resize request, so the correspondence detailed between \
selected region and page size can be lost. This applies to both page \
operations. The scrollbar will resize to show this (if it used to move around \
the file).\n\nR.Expand\nThis locks the 'end selection' datum point to the 'end' \
of the view pane; The 'start' of the view pane moves successively left each \
time 'Right Expand' is clicked. This makes it possible to locate, say, zero-\
crossings at high resolution at both ends of a lengthy file. Zoom and region \
select are used to identify the left-hand selection point which is then \
locked. The region containing the rightmost point of interest can be located \
at any conveniently low reolution and the 'end sel' pointer located somewhere \
after it. 'Right Expand' then increases resolution so that the start is moved \
progressively left, out of the view pane. At any stage the 'end sel' marker \
can be moved leftwards, nearer the point of interest, as the resolution \
increases, so that the desired end point remains withing the view pane.\n\
On each use of 'R.Expand' the end selection value is preserved \
as the 'end' value and located at the right of the view; the new 'start' \
value at the left of the view is half the difference between the current \
start value and the 'end sel' value plus the current start value:\n\
  new start = (end sel - current start)/2 + current start\n\nSel Range\nThis \
causes the section of track in the view pane to become the selected region, \
which is highlighted. It is most useful for file transformation commands \
such as 'Edit / Shift/Scale' and 'Edit / Shape template', when it can be used \
to select the whole length of a track ('All', then 'Sel Range') for \
modification, since the 'Edit' commands work only on selections.\n\n\nScale \
display\n\ny scale\n\
This is a display box giving the factor by which datum values in the view \
pane are scaled. It changes when the mouse is left-clicked and dragged \
vertically to change scale, and it can also accept input to set the scale \
factor to some desired value. Left-clicking the box activates it for text \
entry.\n\n\nLock toggle buttons\n\nlock Left\nLocks the start selection pointer \
at its current position, leaving it unchanged by mouse click, mouse drag or \
function button operation.\n\nlock Right\nLocks the end selection pointer at \
its current position.\n\nSelection locks are particularly useful if the mouse \
needs to be used to drag the file into view or to rescale the view. It is \
also useful in avoiding inadvertent loss of a selected region.\n\nlock Scale\
\nLocks the current y-axis scaling given in the 'y scale' data field, since \
mouse clicks can easily cause this to change if the mouse is not clicked or \
dragged at a vertical postion in the view required to prevent change of \
scaling.\n\nfast Plot\nThis causes points in the data to be skipped if there \
are more datum points than pixels available in the view pane, such that the \
minimum number of data points necessary to construct the view are used. If \
the resolution is such that there is less data to be displayed than pixels \
available, then all the data will be shown in the view. This is the default \
behaviour when 'fast Plot' is not selected (i.e. button raised). This mode \
of operation should be used with extreme caution, particularly with large \
files, since every single datum in the file is plotted in the view. This can \
cause view redraws to take an unreasonably long time, and even longer still \
if a selection exists.\n\n\n3) The scrollbar\nThis \
has three functions, scrolling, file position indicator / memory usage \
indicator and selection indicator. There are also scrollbar right and left \
arrows which appear only when the entire file is not shown in the view \
pane and whose function depends on the selection modeand zoom.\n\n\na) \
Scrolling\nThe scrollbar acts to move around in the file. If \
the position of the scrollbar in the trough does not correspond to a region \
in the view pane, a file load request is issued and the file redisplayed with \
the relevant data foor the scrollbar position added in to the view of the \
data already displayed. The position of the scrollbar may not always be \
updated for file drag or button operations, and the bar should be left-\
clicked to ensure it is in a position corresponding to the data in the view \
pane. Note that this may cause the scrollbar to jump position.\n\
Left-clicking at either side of the scrollbar in the trough causes the scroll\
bar to jump by a 'page' left or right. If there is no selection in the file \
(the scrollbar trough is not highlighted) then the page size remains constant \
and successive left-clicks in the scrollbar trough cause the view to move \
forward or back by this constant amount.\n\
If a region is selected (scrollbar trough highlighted) the page size has a \
magnitude dependendent on how much \
of the file is loaded which in turn depends on the scrollbar itself. Thus \
successive clicks not only cause the view to scroll forwards or back but also \
the size of the data loaded to change, reflected in the scrollbar dimensions. \
Successive jumps will not be in equal increments.\nIn both cases \
simultaneous left+right-\
clicking (or middle mouse button) causes the scrollbar to move so that its \
centre is at the postion clicked in the scrollbar trough; this can be \
used to rapidly jump from one end of a file to the other.\n\nScrollbar \
arrows\nIf there is no selected region, the arrowheads initally move the \
scrollbar within the region \
displayed; subsequent left-clicks on either arrow move the view one sample \
left or right.\nIf a selected region exists and is within the view range \
but does not occupy its full extent, the behaviour is as for 'scrollbar \
unselected' mode as before: left arrow shifts the view pane right one \
sample and right arrow shifts the view left one sample.\n\
If 'Zoom in' is used so the selection occupies the whole view pane then \
the arrow behaviour is different: left arrow displays more of the file before \
the selection start whilst the end selection remains fixed at the right-hand \
side of the view pane; right arrow moves the view forwards in the file until \
the left selection reaches the left-hand side of the view pane, and then locks \
at this position whilst increasing the amount of the file displayed after \
the right selection point.\n\nb) File \
pointer\nThe size of the scrollbar and its position in the trough give an \
indication of the amount of the file currently in the view pane with respect \
to the total file size (the length of the trough) and also the location of \
the data currently in the view in the file. This is the cause of the \
apparently erratic behaviour noted above: the scrollbar position may not \
correspond to the currently loaded section of file; left-clicking confirms \
this correspondence.\nThe size of the scrollbar is \
also an indication of the amount of \
memory used.\n\nc) Selection indicator\nThe scrollbar trough colour indicates the presence or absence \
of a selection in the file; left-click the bar to confirm its current state. \
When the trough remains highlighted, a selection exists in the file.\n\n\n\
4) The data display pane\nThis selection of interactive data fields displays \
numerical information about the file. Left-clicking a field allows data \
entry. When data is entered it should be confirmed by pressing 'Return'.\n\n\
Start / End\nThese display the amount of the file currently \
loaded into memory in terms of starting sample datum number and end datum \
number. Changing the data in these fields causes a redisplay of the data \
in the range chosen.\n\nStart sel / End sel\nThis gives the start and end \
datum numbers of a selected region. If no selected region exists, these \
values will be the same. A region may be selected either with mouse left-\
click and drag, or by manually entering the desired start and end into the \
selection start and end fields.\nNote that selecting a range outside of the \
data currently in the view pane does not result in the view being updated. \
The region selected may not even be visible unless it at least partly lies \
within the existing view.\n\nSample\nGives \
the datum number (current position in the file in terms of data items \
contained in it) pointed to by the mouse. If the mouse is not within the view \
pane, this value can be set manually to find the datum value at the specified \
position.\n\nMagnitude\nThis displays the current value of the datum whose \
position is given by the 'sample' field. If the mouse does not lie within the \
view pane then this can be used to change the value of a single datum.\n\n\n\
The supplementary data display\nThis only appears if a file of type wave \
is displayed in the viewer. It displays the same information as given in \
the viewer data display, but in terms of time rather than sample number.\n\
The fields are interactive and cross-linked to the viewer data display so \
that changes in the fields in one display are reflected in the other.\n\
Note that the format used for data display and entry is\n\n  mins:decimal \
secs\n\n\
The decimal point may be omitted (e.g. m:s and not m:s.s) but the colon used \
as separator is mandatory unless only minutes are specified (e.g. '5' \
specifies '5:0.0'). Values entered into the display fields should be confirmed \
by pressing 'Return'.\nThe number of decimal points displayed is dependent on \
Options / Display / FP precision\n\nNote that the '%peak' data field allows \
specification of a datum value by reference to the peak values allowed for \
the file type as specified in Options / Display / INT range.\n\nThe final \
field in the supplementary data display is the duration of the selected \
region, which has no correspondence with the viewer (there is no equivalent \
item giving number of samples selected since this is redundant and likely of \
little use).");

	_edit->setEditStatus ( False );
}


void ViewerHelpCmd::okCallback ( XtPointer clientData, char * text )
{
	ViewerHelpCmd *obj = ( ViewerHelpCmd * ) clientData;

	obj->ok( clientData );	// ok mapped to 'enter'
}


void ViewerHelpCmd::cancelCallback ( XtPointer clientData )
{
	ViewerHelpCmd *obj = ( ViewerHelpCmd * ) clientData;

	obj->ok( clientData );	// call ok to delete object
}


void ViewerHelpCmd::ok ( void *clientData )
{
	ViewerHelpCmd *obj = ( ViewerHelpCmd * ) clientData;

	if (obj->_editRealised == TRUE )
	{
		obj->_editRealised = FALSE;
		delete obj->_edit;
	}
}
